<?php namespace CodeSpace\API;

class API {

	public array $datatypes;
	public array $map;

	function __construct() {

		$this->map = [];

	}

	public function convertResponse(mixed $res): array {
		if ($res === null) {
			$res = ["error" => "invalid_endpoint"];
		}
		if (is_bool($res)) {
			$res = ["success" => $res];
		}
		if (!is_array($res)) $res = [];
		if (array_key_exists("error", $res)) $res["success"] = false;
		return $res;
	}

	function add(array $path, string $class) {
		$root = &$this->map;
		foreach ($path as $part) {
			if (!array_key_exists($part, $root)) {
				$root[$part] = [];
			}
			$root = &$root[$part];
		}
		$root["__class"] = $class;
	}

	function getClass(array $path): string | null {
		$root = &$this->map;
		foreach ($path as &$part) {
			if (!array_key_exists($part, $root)) return null;
			$root = &$root[$part];
		}
		return array_key_exists("__class", $root) ? $root["__class"] : null;
	}

	function getEndpoint(string $class): Endpoint {
		return new $class($this);
	}

	function getAction(): array {
		$action = null;
		if (array_key_exists("action", $_REQUEST)) {
			$action = $_REQUEST["action"];
		}
		if ($action == null) {
			$action = substr(parse_url($_SERVER["REQUEST_URI"], PHP_URL_PATH), 1);
		}
		return explode("/", $action);
	}

	function execute(array $path, array $data): mixed {
		$class = $this->getClass($path);
		if ($class == null) return null;
		$endpoint = $this->getEndpoint($class);
		return $endpoint->get($data);
	}

	function run() {
		header("Content-Type: application/json");
		$response = $this->execute($this->getAction(), $_REQUEST);
		echo json_encode($this->convertResponse($response));
		exit();
	}

}