<?php namespace CodeSpace\API;

use Exception;

abstract class Endpoint {

	private API $api;

	protected ?array $params = null;

	public function __construct(API $api) {
		$this->api = $api;
	}

	public function get(array $data): array {
		$params = Parser::parse($data, $this->params);
		if ($params !== null) {
			try {
				$res = $this->run($params);
			} catch(Exception $e) {
				$res = ["error" => "internal", "detailed_error" => $e->getMessage()];
			}
		} else {
			$res = ["error" => "missing_data"];
		}
		return $this->api->convertResponse($res);
	}

	abstract public function run(array $params);

}